"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var t = __importStar(require("@babel/types"));
var ast_builders_1 = require("../utils/ast-builders");
var css_builders_1 = require("../utils/css-builders");
/**
 * Interrogates `node` and returns styled data if any were found.
 * @param node
 */
var extractStyledDataFromNode = function (node, meta) {
    var _a, _b;
    if (t.isTaggedTemplateExpression(node) &&
        t.isMemberExpression(node.tag) &&
        t.isIdentifier(node.tag.object) &&
        node.tag.object.name === ((_a = meta.state.compiledImports) === null || _a === void 0 ? void 0 : _a.styled) &&
        t.isIdentifier(node.tag.property)) {
        var tagName = node.tag.property.name;
        var cssNode = node.quasi;
        return {
            tagName: tagName,
            cssNode: cssNode,
        };
    }
    if (t.isCallExpression(node) &&
        t.isMemberExpression(node.callee) &&
        t.isIdentifier(node.callee.object) &&
        node.callee.object.name === ((_b = meta.state.compiledImports) === null || _b === void 0 ? void 0 : _b.styled) &&
        t.isExpression(node.arguments[0]) &&
        t.isIdentifier(node.callee.property)) {
        var tagName = node.callee.property.name;
        var cssNode = node.arguments[0];
        return {
            tagName: tagName,
            cssNode: cssNode,
        };
    }
    return undefined;
};
/**
 * Takes a styled tagged template or call expression and then transforms it to a compiled component.
 *
 * `styled.div({})`
 *
 * @param path Babel path - expects to be a tagged template or call expression.
 * @param state Babel state - should house options and meta data used during the transformation.
 */
exports.visitStyledPath = function (path, meta) {
    var styledData = extractStyledDataFromNode(path.node, meta);
    if (!styledData) {
        // We didn't find a node we're interested in - bail out!
        return;
    }
    var cssOutput = css_builders_1.buildCss(styledData.cssNode, meta);
    path.replaceWith(ast_builders_1.buildStyledComponent(__assign(__assign({}, meta.state.opts), { cssOutput: cssOutput, tagName: styledData.tagName, parentPath: path, scope: path.scope })));
};
//# sourceMappingURL=index.js.map