import * as t from '@babel/types';
import { NodePath } from '@babel/traverse';
import { Metadata } from '../types';
/**
 * Returns the nodes path including the scope of a parent.
 * @param node
 * @param parentPath
 */
export declare const getPathOfNode: <TNode extends {}>(node: TNode, parentPath: NodePath<t.Node>) => NodePath<TNode>;
/**
 * Returns the binding identifier for a member expression.
 * For example the member expression `foo.bar.baz` will return the `foo` identifier.
 *
 * @param expression - Member expression node.
 */
export declare const getMemberExpressionMeta: (expression: t.MemberExpression) => {
    bindingIdentifier: t.Identifier;
    accessPath: t.Identifier[];
};
/**
 * Will return the value of a path from an object expression.
 *
 * For example if  we take an object expression that looks like:
 * ```
 * { colors: { primary: 'red' } }
 * ```
 *
 * And a path of identifiers that looks like:
 * ```
 * [colors, primary]
 * ```
 *
 * Would result in returning the `red` string literal node.
 * If the value is not found `undefined` will be returned.
 *
 * @param expression - Member expression node.
 * @param accessPath - Access path identifiers.
 */
export declare const getValueFromObjectExpression: (expression: t.ObjectExpression, accessPath: t.Identifier[]) => t.AnyTypeAnnotation | t.ArgumentPlaceholder | t.ArrayExpression | t.ArrayPattern | t.ArrayTypeAnnotation | t.ArrowFunctionExpression | t.AssignmentExpression | t.AssignmentPattern | t.AwaitExpression | t.BigIntLiteral | t.BinaryExpression | t.LogicalExpression | t.BindExpression | t.BlockStatement | t.Program | t.TSModuleBlock | t.CatchClause | t.DoWhileStatement | t.ForInStatement | t.ForStatement | t.FunctionDeclaration | t.FunctionExpression | t.ObjectMethod | t.SwitchStatement | t.WhileStatement | t.ForOfStatement | t.ClassMethod | t.ClassPrivateMethod | t.BooleanLiteral | t.BooleanLiteralTypeAnnotation | t.BooleanTypeAnnotation | t.BreakStatement | t.CallExpression | t.ClassExpression | t.ClassDeclaration | t.ClassBody | t.ClassImplements | t.ClassPrivateProperty | t.ClassProperty | t.ContinueStatement | t.ReturnStatement | t.ThrowStatement | t.ConditionalExpression | t.IfStatement | t.DebuggerStatement | t.DecimalLiteral | t.VariableDeclaration | t.ExportAllDeclaration | t.ExportDefaultDeclaration | t.ExportNamedDeclaration | t.ImportDeclaration | t.DeclareClass | t.DeclareFunction | t.DeclareInterface | t.DeclareModule | t.DeclareModuleExports | t.DeclareTypeAlias | t.DeclareOpaqueType | t.DeclareVariable | t.DeclareExportDeclaration | t.DeclareExportAllDeclaration | t.InterfaceDeclaration | t.OpaqueType | t.TypeAlias | t.EnumDeclaration | t.TSDeclareFunction | t.TSInterfaceDeclaration | t.TSTypeAliasDeclaration | t.TSEnumDeclaration | t.TSModuleDeclaration | t.DeclaredPredicate | t.Decorator | t.Directive | t.DirectiveLiteral | t.DoExpression | t.EmptyStatement | t.EmptyTypeAnnotation | t.EnumBooleanBody | t.EnumNumberBody | t.EnumStringBody | t.EnumSymbolBody | t.EnumBooleanMember | t.EnumDefaultedMember | t.EnumNumberMember | t.EnumStringMember | t.ExistsTypeAnnotation | t.ExportDefaultSpecifier | t.ExportNamespaceSpecifier | t.ExportSpecifier | t.Identifier | t.StringLiteral | t.NumericLiteral | t.NullLiteral | t.RegExpLiteral | t.MemberExpression | t.NewExpression | t.ObjectExpression | t.SequenceExpression | t.ParenthesizedExpression | t.ThisExpression | t.UnaryExpression | t.UpdateExpression | t.MetaProperty | t.Super | t.TaggedTemplateExpression | t.TemplateLiteral | t.YieldExpression | t.Import | t.OptionalMemberExpression | t.OptionalCallExpression | t.TypeCastExpression | t.JSXElement | t.JSXFragment | t.PipelinePrimaryTopicReference | t.RecordExpression | t.TupleExpression | t.TSAsExpression | t.TSTypeAssertion | t.TSNonNullExpression | t.ExpressionStatement | t.File | t.NullLiteralTypeAnnotation | t.FunctionTypeAnnotation | t.FunctionTypeParam | t.GenericTypeAnnotation | t.InferredPredicate | t.InterfaceExtends | t.InterfaceTypeAnnotation | t.IntersectionTypeAnnotation | t.MixedTypeAnnotation | t.NullableTypeAnnotation | t.NumberLiteralTypeAnnotation | t.NumberTypeAnnotation | t.ObjectTypeAnnotation | t.ObjectTypeInternalSlot | t.ObjectTypeCallProperty | t.ObjectTypeIndexer | t.ObjectTypeProperty | t.ObjectTypeSpreadProperty | t.QualifiedTypeIdentifier | t.StringLiteralTypeAnnotation | t.StringTypeAnnotation | t.SymbolTypeAnnotation | t.ThisTypeAnnotation | t.TupleTypeAnnotation | t.TypeofTypeAnnotation | t.TypeAnnotation | t.TypeParameter | t.TypeParameterDeclaration | t.TypeParameterInstantiation | t.UnionTypeAnnotation | t.Variance | t.VoidTypeAnnotation | t.JSXAttribute | t.JSXClosingElement | t.JSXExpressionContainer | t.JSXSpreadChild | t.JSXOpeningElement | t.JSXText | t.JSXOpeningFragment | t.JSXClosingFragment | t.ImportAttribute | t.ImportDefaultSpecifier | t.ImportNamespaceSpecifier | t.ImportSpecifier | t.InterpreterDirective | t.JSXEmptyExpression | t.JSXIdentifier | t.JSXMemberExpression | t.JSXNamespacedName | t.JSXSpreadAttribute | t.RestElement | t.ObjectPattern | t.TSParameterProperty | t.LabeledStatement | t.Noop | t.ObjectProperty | t.PipelineBareFunction | t.PipelineTopicExpression | t.Placeholder | t.PrivateName | t.SpreadElement | t.TryStatement | t.WithStatement | t.TSImportEqualsDeclaration | t.TSExportAssignment | t.TSNamespaceExportDeclaration | t.SwitchCase | t.TSAnyKeyword | t.TSArrayType | t.TSBooleanKeyword | t.TSBigIntKeyword | t.TSNeverKeyword | t.TSNullKeyword | t.TSNumberKeyword | t.TSObjectKeyword | t.TSStringKeyword | t.TSSymbolKeyword | t.TSUndefinedKeyword | t.TSUnknownKeyword | t.TSVoidKeyword | t.TSThisType | t.TSLiteralType | t.TSCallSignatureDeclaration | t.TSConditionalType | t.TSConstructSignatureDeclaration | t.TSConstructorType | t.TSDeclareMethod | t.TSQualifiedName | t.TSEnumMember | t.TSExpressionWithTypeArguments | t.TSExternalModuleReference | t.TSFunctionType | t.TSImportType | t.TSIndexSignature | t.TSIndexedAccessType | t.TSInferType | t.TSInterfaceBody | t.TSIntersectionType | t.TSMappedType | t.TSMethodSignature | t.TSNamedTupleMember | t.TSOptionalType | t.TSParenthesizedType | t.TSPropertySignature | t.TSRestType | t.TSTupleType | t.TSTypeReference | t.TSTypePredicate | t.TSTypeQuery | t.TSTypeLiteral | t.TSUnionType | t.TSTypeOperator | t.TSTypeAnnotation | t.TSTypeParameter | t.TSTypeParameterDeclaration | t.TSTypeParameterInstantiation | t.TemplateElement | t.V8IntrinsicIdentifier | t.VariableDeclarator | undefined;
/**
 * Will return either the name of an identifier or the value of a string literal.
 *
 * E.g:
 * - `foo` identifier node will return `"foo"`,
 * - `"bar"` string literal node will return `"bar"`.
 *
 * @param node
 */
export declare const getKey: (node: t.Expression) => string;
/**
 * Will traverse a path and its identifiers to find all bindings.
 * If any of those bindings are mutated `true` will be returned.
 *
 * @param path
 */
export declare const isPathReferencingAnyMutatedIdentifiers: (path: NodePath<any>) => boolean;
/**
 * Will try to statically evaluate the node.
 * If successful it will return a literal node,
 * else it will return the fallback node.
 *
 * @param node Node to evaluate
 * @param meta
 * @param fallbackNode Optional node to return if evaluation is not successful. Defaults to `node`.
 */
export declare const tryEvaluateExpression: (node: t.Expression, meta: Metadata, fallbackNode?: t.Expression) => t.Expression;
/**
 * Will look in an expression and return the actual value.
 * If the expression is an identifier node (a variable) and a constant,
 * it will return the variable reference.
 *
 * E.g: If there was a identifier called `color` that is set somewhere as `const color = 'blue'`,
 * passing the `color` identifier to this function would return `'blue'`.
 *
 * This behaviour is the same for const string & numeric literals,
 * and object expressions.
 *
 * @param expression Expression we want to interrogate.
 * @param state Babel state - should house options and meta data used during the transformation.
 */
export declare const getInterpolation: (expression: t.Expression, meta: Metadata) => t.Expression;
/**
 * Will wrap BlockStatement or Expression in an IIFE,
 * Looks like (() => { return 10; })().
 *
 * @param node Node of type either BlockStatement or Expression
 */
export declare const wrapNodeInIIFE: (node: t.ArrayExpression | t.ArrowFunctionExpression | t.AssignmentExpression | t.AwaitExpression | t.BigIntLiteral | t.BinaryExpression | t.LogicalExpression | t.BindExpression | t.BlockStatement | t.FunctionExpression | t.BooleanLiteral | t.CallExpression | t.ClassExpression | t.ConditionalExpression | t.DecimalLiteral | t.DoExpression | t.Identifier | t.StringLiteral | t.NumericLiteral | t.NullLiteral | t.RegExpLiteral | t.MemberExpression | t.NewExpression | t.ObjectExpression | t.SequenceExpression | t.ParenthesizedExpression | t.ThisExpression | t.UnaryExpression | t.UpdateExpression | t.MetaProperty | t.Super | t.TaggedTemplateExpression | t.TemplateLiteral | t.YieldExpression | t.Import | t.OptionalMemberExpression | t.OptionalCallExpression | t.TypeCastExpression | t.JSXElement | t.JSXFragment | t.PipelinePrimaryTopicReference | t.RecordExpression | t.TupleExpression | t.TSAsExpression | t.TSTypeAssertion | t.TSNonNullExpression) => t.CallExpression;
/**
 * Will pick `ArrowFunctionExpression` body and tries to wrap it in an IIFE if
 * its a BlockStatement otherwise returns the picked body,
 * E.g.
 * `props => props.color` would end up as `props.color`.
 * `props => { return props.color` } would end up as `(() => { return props.color })()`.
 *
 * @param node Node of type ArrowFunctionExpression
 */
export declare const pickArrowFunctionExpressionBody: (node: t.ArrowFunctionExpression) => t.Expression;
